/*
  dirdialog.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "dirdialog.h"

#include <stdlib.h>

#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qhbox.h>
#include <qsizepolicy.h>
#include <qlineedit.h>
#include <qstringlist.h>
#include <qdir.h>
#include <qmessagebox.h>
#include <qheader.h>

#include <qlistview.h>

DirDialog::DirDialog(QWidget* parent, QString const &caption, QString const &path)
	: QDialog( parent, "dirdialog", TRUE, 0 )
{
	if (g_machine.resolution() == QVGA) {
		resize(200, 200);
	} else {
		resize(400, 400);
		setFont(QFont("lcfont", 16));
	}

	setCaption(caption);

#ifdef QWS
	home_path = QDir::homeDirPath() + "/Documents";
#else
	home_path = QDir::homeDirPath();
	setFont(QFont("gothic", 12));
#endif

	base_vbox = new QVBoxLayout(this);

	/* create directory tree */
	dirtree = new QListView(this);
	dirtree->addColumn(QString::fromUtf8("text"));
	dirtree->addColumn(QString::fromUtf8("path"));
	dirtree->setColumnWidthMode(0, QListView::Maximum);
	dirtree->setColumnWidthMode(1, QListView::Manual);
	dirtree->setColumnWidth(1,0);
	dirtree->header()->hide();
	dirtree->setSorting(1);
	dirtree->setRootIsDecorated(TRUE);

	base_vbox->addWidget(dirtree);
	base_vbox->addSpacing(3);

	/* button box */
	button_hbox = new QHBox(this);
	button_hbox->setSpacing(3);
	cancel_button = new QPushButton("Cancel", button_hbox);
	ok_button = new QPushButton("OK", button_hbox);

	base_vbox->addWidget(button_hbox);

	/* connect signal/slot */
	connect(dirtree, SIGNAL(expanded(QListViewItem *)),
			this, SLOT(listExpanded(QListViewItem *)));

	connect(ok_button, SIGNAL(clicked()),
			this, SLOT(accept()));
	connect(cancel_button, SIGNAL(clicked()),
			this, SLOT(reject()));

	/* init listview */
	QListViewItem *item = new QListViewItem(dirtree, QString::fromUtf8("本体メモリ"), home_path);
	item->setPixmap(0, QPixmap(IMG_MAIN));
	emptyItem(item);
	dirtree->insertItem(item);

	if(!system("mount | grep -q cf")){
		item = new QListViewItem(dirtree, QString::fromUtf8("CompactFlash"), CF_PATH);
		item->setPixmap(0, QPixmap(IMG_CF));
		emptyItem(item);
		dirtree->insertItem(item);
	}

	if(!system("mount | grep -q card")){
		item = new QListViewItem(dirtree, QString::fromUtf8("SDカード"), CARD_PATH);
		item->setPixmap(0, QPixmap(IMG_SD));
		emptyItem(item);
		dirtree->insertItem(item);
	}

	item = new QListViewItem(dirtree, QString::fromUtf8("FileSystem"), ALL_PATH);
	item->setPixmap(0, QPixmap(IMG_ALL));
	emptyItem(item);
	dirtree->insertItem(item);

	if(path != "")
		setPath(path);
}

DirDialog::~DirDialog()
{
}

QString DirDialog::getPath()
{
	return dirtree->currentItem()->text(1);
}

void DirDialog::listExpanded(QListViewItem *item)
{
	QListViewItem *child = item->firstChild();

	if(!child || child->text(1) != "...")
		return;

	delete child;

	// load directory list
	QDir d(item->text(1));
	d.setFilter(QDir::Dirs);
	d.setSorting(QDir::Name);

	if(!d.exists()){
		QMessageBox::critical(this,
							  "Error!!",
							  item->text(1)
							  + QString::fromUtf8("が見つかりません"));
		return;
	}

	const QFileInfoList *list = d.entryInfoList();
	QFileInfoListIterator it( *list );
	QFileInfo *fi;

	while ( (fi=it.current()) ) {
		++it;
		if(fi->fileName() == "." || fi->fileName() == "..")
			continue;

		child = new QListViewItem(item, fi->fileName(), item->text(1)+"/"+fi->fileName());
		child->setPixmap(0, QPixmap(IMG_DIR));
		emptyItem(child);
	}
}

void DirDialog::emptyItem(QListViewItem *item)
{
	new QListViewItem(item, EMPTY_MARK, EMPTY_MARK);
}

void DirDialog::setPath(QString const &path)
{
    QListViewItem *it;
	QString pathstr = path;

	while(pathstr[pathstr.length()-1] == '/')
		pathstr.truncate(pathstr.length()-1);

	it = dirtree->firstChild();
	/* loop for depth */
	while(it){
		/* loop for width */
		while (it) {
			/* matched perfectory */
			if(it->text(1) == pathstr){
				dirtree->setCurrentItem(it);
				dirtree->ensureItemVisible(it);
				return;
			}

			/* matched partially */
			if(pathstr.find(it->text(1)+"/") == 0)
				break;

			it = it->nextSibling();
		}

		/* not found */
		if(!it)
			return;

		/* expand (with load) item */
		it->setOpen(true);
		it = it->firstChild();
	}
}
