/*
  map_grid.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG
#include "qgmap.h"
#include <qpixmap.h>
#include "pos.h"
#include "map.h"
#include "map_grid.h"

MapItemGrid::MapItemGrid(Map *parent)
{
	parent->addItem(this);

	connect(parent, SIGNAL(configChanged(const QString &, const QString &)),
			this, SLOT(setConfig(const QString &, const QString &)));

	int i;
	for (i = 0; i < 10; i++)
		number[i].load(QString(APP_PATH "/%1.png").arg(i));

	number[10].load(APP_PATH "/x.png");
	number[11].load(APP_PATH "/y.png");
	number[12].load(APP_PATH "/z.png");

	enable = 0;
}

MapItemGrid::~MapItemGrid()
{
}

bool MapItemGrid::drawItem(QImage &scr, const Pos &pos)
{
	if (!enable)
		return false;

	Enter();

	drawImageString(scr, QString("a%1").arg(pos.getX() / IMAGE_SIZE), 5, 5);
	drawImageString(scr, QString("b%1").arg(pos.getY() / IMAGE_SIZE), 5, 5 + 18);
	drawImageString(scr, QString("c%1").arg(pos.getZoom()), 5, 5 + 36);

	int i;

	unsigned int *x = (unsigned int *)scr.scanLine(0);
	for (i = 0; i < IMAGE_SIZE; i++) {
		*x++ = 0xff000000;
		unsigned int *p = (unsigned int *)scr.scanLine(i);
		*p = 0xff000000;
	}

	return true;
}

void MapItemGrid::drawImageString(QImage &scr, const QString &str, int x, int y)
{
	int i;
	for (i = 0; i < (int)str.length(); i++) {
		QImage *img = &number[str.mid(i,1).toInt(0, 16)];
		drawImage(scr, img, x, y);
		x += img->width();
	}
}

void MapItemGrid::setConfig(const QString &key, const QString &value)
{
	Enter();

	if(key=="Grid"){
		enable = value.toInt();
//		emit reload(true);
	}
}

void MapItemGrid::request(QString const &req, QString const &arg)
{
	if(req == "Grid" && arg == "Toggle") {
		enable = !enable;
//		emit reload(true);
	}
}
