/*
  poi_search_dialog.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "poi_search_dialog.h"
#include "mainwindow.h"
#include "mainwidget.h"
#include <qstring.h>
#include <qlabel.h>
#include <qgrid.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qvgroupbox.h>
#include <qpushbutton.h>
#include <qsizepolicy.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qmessagebox.h>
#include "poi_db.h"
#include "progressdialog.h"
#include <stdio.h>
#include <qapplication.h>
#include <unistd.h>
#include <qlineedit.h>
#include <qcombobox.h>

#define DIALOG_WIDTH  640
#define DIALOG_HEIGHT 640

PoiSearchDialog::PoiResultItem::PoiResultItem(QListView * parent) :
	QListViewItem(parent)
{
}

QString PoiSearchDialog::PoiResultItem::key(int column, bool ascending) const
{
	if (column == 0){
		return distance;
	}
	return QListViewItem::key(column, ascending);
}

void PoiSearchDialog::PoiResultItem::setDistance(int distance)
{
	this->distance.sprintf("%010d", distance);
}

PoiSearchDialog::PoiSearchDialog( QWidget* parent, const char* name,
								bool modal, WFlags fl)
	: QDialog( parent, name, modal, fl )
{
    resize(pMainWindow->size());

	setCaption(QString::fromUtf8("POI検索"));

	QFont small_font = font();
	small_font.setPointSize(small_font.pointSize() * 3 / 4);

	QVBoxLayout *base_box = new QVBoxLayout(this);

	/* 検索条件 frame */
	QVGroupBox *search_frame = new QVGroupBox(QString::fromUtf8("検索条件"),this);
	base_box->addWidget(search_frame);

	QGrid *search_grid = new QGrid(2, search_frame);

	/* 検索キーワード */
	new QLabel(QString::fromUtf8("検索:"), search_grid);

	QHBox *keyword_box = new QHBox(search_grid);
	keyword = new QLineEdit(keyword_box);
	QPushButton *search_button = new QPushButton(QString::fromUtf8("検索"),
												 keyword_box);
	connect(search_button, SIGNAL(clicked()), this, SLOT(press_search()));
	connect(keyword, SIGNAL(returnPressed()), this, SLOT(press_search()));

	/* ファイル選択 */
	QVBox *vbox = new QVBox(search_grid);
	new QLabel(QString::fromUtf8("対象:"), vbox);
	new QWidget(vbox);
//	vbox->setStretchFactor(dummy, 10);

	filelistview = new QListView(search_grid);
	filelistview->addColumn(QString::fromUtf8("対象"));
	filelistview->addColumn(QString::fromUtf8("名前"));
	filelistview->setAllColumnsShowFocus(true);
	filelistview->setColumnAlignment(0, Qt::AlignHCenter);
	filelistview->setMaximumHeight(90);
	filelistview->setFont(small_font);

	/* 検索結果 frame */
	QVGroupBox *result_frame = new QVGroupBox(QString::fromUtf8("検索結果"),this);
	base_box->addWidget(result_frame, 10);
	result = new QListView(result_frame);
	result->addColumn(QString::fromUtf8("距離"));
	result->addColumn(QString::fromUtf8("名前"));
	result->addColumn(QString::fromUtf8("説明"));
	result->setAllColumnsShowFocus(true);
	result->setFont(small_font);

	connect(filelistview, SIGNAL(clicked(QListViewItem *, const QPoint &, int)),
			this, SLOT(click_row(QListViewItem *, const QPoint &, int)));

#if 0
	/* ボタン Box */
	QHBox *button_box = new QHBox(this);
	base_box->addWidget(button_box, 0);
	QWidget *space = new QWidget(button_box);
	button_box->setStretchFactor(space, 10);

	combo = new QComboBox(FALSE, button_box);
	combo->insertItem(QString::fromUtf8("移動"));
	for (int i = 1; i <= 10; i++) {
		combo->insertItem(QString::fromUtf8("マーカー%1に設定").arg(i % 10));
	}
	QPushButton *exec = new QPushButton(QString::fromUtf8("実行"),
										button_box);

#ifndef QWS
	QPushButton *cancel = new QPushButton("Cancel", button_box);
	QPushButton *ok = new QPushButton("OK", button_box);

	ok->setDefault(true);

	connect(cancel, SIGNAL(clicked()),
			this, SLOT(reject()));
	connect(ok, SIGNAL(clicked()),
			this, SLOT(accept()));
#endif
#endif

	db = new PoiDb(g_db_poi);

	check[0].load(APP_PATH "/check-none.png");
	check[1].load(APP_PATH "/check.png");

	arrow.load(ARROW_FILE);

	updateFileItem();
	updateGeometry();

	MainWidget *main = (MainWidget *)lookupObject(this, "MainWidget", "mainwidget");
	center = main->getPosition();
	printf("center %lf,%lf\n", center.getLat(), center.getLon());
}

PoiSearchDialog::~PoiSearchDialog()
{
	delete db;
}

#if 0
void PoiSearchDialog::updateProgress(int now, int max)
{
	if (!progress)
		return;

	if (progress_timer.elapsed() > 100) {
		progress->setMax(max);
		progress->setStep(QString::fromUtf8("登録中・・・"), now);
		progress_timer.restart();
	}
}

void PoiSearchDialog::importProgress(int now, int max, void *data)
{
	PoiSearchDialog *myself = (PoiSearchDialog *)data;

	myself->updateProgress(now, max);
}
#endif

void PoiSearchDialog::press_search()
{
	QString word = keyword->text();

	if (word.length() == 0)
		return;

	updateResult(word);
}

void PoiSearchDialog::updateFileItem()
{
	filelistview->clear();

	PoiDbFile *file = db->queryPoiFile(0);
	if (!file)
		return;

	int i = 0;

	if (!file->next()) {
		delete file;
		return;
	}

	file_info.clear();

	do {
		QListViewItem *item = new QListViewItem(filelistview);
		struct poi_file_info info;

		info.enable = 1;
		info.id = file->getId();
		file_info.insert(item, info);

		item->setPixmap(0, check[file_info[item].enable]);
		item->setText(1, file->getName());

		item->setPixmap(1, QPixmap(db->getPoiIcon(file->getIconId())));
		filelistview->insertItem(item);
#if 0
		printf("%d %d %d %d %s\n",
			   file->getId(), file->getGrpId(), file->getIconId(),
			   file->getEnabled(),
			   (const char *)file->getName().utf8());
#endif
		i++;
	} while(file->next());
	delete file;
}

void PoiSearchDialog::click_row(QListViewItem *item,
								const QPoint &point,
								int column)
{
	UNUSED(point);
	UNUSED(column);

	if (!item)
		return;

	int enable = file_info[item].enable;
	file_info[item].enable = !enable;
	item->setPixmap(0, check[!enable]);
}

void PoiSearchDialog::updateResult(QString const &keyword)
{
	result->clear();

	int count = db->countItem(keyword);

	if (count > MAX_RESULT) {
		QMessageBox::critical(this, QString::fromUtf8("検索エラー"),
							  QString::fromUtf8("結果が %1 件を超えています (%2件)").arg(MAX_RESULT).arg(count),
							  QMessageBox::Ok, 0, 0);
		return;
	}

	PoiDbItem *res = db->searchItem(keyword);
	if (!res)
		return;

	result_info.clear();

	while (res->next()) {
		PoiResultItem *item = new PoiResultItem(result);
//		QListViewItem *item = new QListViewItem(filelistview);
		QString name = res->getName();
		QString info = res->getInfo();
		int j;

		while ((j = name.find('\n')) >= 0)
			name[j] = ' ';

		while ((j = info.find('\n')) >= 0)
			info[j] = ' ';

		result_info[item] = res->getId();

//		printf("item:%p id:%d\n", item, res->getId());

		Pos pos(res->getX(), res->getY(), 0);
		double dist = center.distance(pos);
		int index = (center.direction_deg(pos) + 360.0/32.0) / (360.0/16.0);
		index %= 16;
		QString str;
		QPixmap pix(16, 16);

		if (dist > 100000.0) {
			str = QString::number((int)(dist / 1000.0)) + "km";
		} else if (dist > 1000.0) {
			str = QString::number(dist / 1000.0, 'f', 1) + "km";
		} else {
			str = QString::number((int)dist) + "m";
		}

		bitBlt(&pix, QPoint(0, 0),
			   &arrow, QRect(16 * index, 0, 16, 16));

		item->setText(0, str);
		item->setDistance(dist);
		item->setPixmap(0, pix);
		item->setPixmap(1, QPixmap(db->getPoiIcon(res->getIconId())));
		item->setText(1, name);
		item->setText(2, info);
	}
	delete res;
}

void PoiSearchDialog::accept(void)
{
	if (keyword->hasFocus()) {
		press_search();
		return;
	}

	if (result->selectedItem()) {
		QDialog::accept();
		return;
	}
}

Pos PoiSearchDialog::getPos()
{
	int x, y;

	if (!result->currentItem())
		return Pos();

	PoiDbItem *item = db->queryPoiItem(result_info[result->currentItem()]);

	if (!item)
		return Pos();

	if (!item->next())
		return Pos();

	x = item->getX();
	y = item->getY();

	delete item;

	MainWidget *main = (MainWidget *)lookupObject(this, "MainWidget", "mainwidget");

	Pos pos(x, y, 0);
	pos.zoomTo(main->getPosition().getZoom());

	return pos;
}
