/*
  poiinfoview.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "poiinfoview.h"
#include <qlabel.h>
#include <qpixmap.h>
#include <qregexp.h>
#include <stdio.h>

PoiInfoView::PoiInfoView(QWidget *parent, const char *name, WFlags f) :
	QVBox(parent, name, f)
{
	QVBox::hide();

	if (g_machine.type() == SL_A300) {
		setFixedWidth(200);
		setFixedHeight(100);
	} else {
		setFixedWidth(300);
		setFixedHeight(130);
		setFont(QFont("lcfont", 14));
	}

	setFrameStyle(QFrame::Box | QFrame::Plain);
#ifndef QWS
	setFont(QFont("gothic", 8));
#endif
	setMargin(2);
	setSpacing(5);

	QHBox *title_hbox = new QHBox(this);
	icon = new QLabel(title_hbox);
	icon->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	icon->setFixedSize(40, 30);
	title = new QLabel(title_hbox);
	title->setAlignment(Qt::AlignLeft | Qt::AlignVCenter | Qt::WordBreak);

	QHBox *desc_hbox = new QHBox(this);
	QLabel *d_label = new QLabel(QString::fromUtf8("説明"), desc_hbox);
	d_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter | Qt::WordBreak);
	d_label->setFixedWidth(40);
	desc = new QLabel(desc_hbox);
	desc->setAlignment(Qt::AlignLeft | Qt::AlignVCenter | Qt::WordBreak);

	QHBox *info_hbox = new QHBox(this);
	QLabel *f_label = new QLabel(QString::fromUtf8("File"), info_hbox);
	f_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter | Qt::WordBreak);
	f_label->setFixedWidth(40);
	info = new QLabel(info_hbox);

	dummy = new QLabel(this);
	dummy->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
}

void PoiInfoView::resizeEvent(QResizeEvent * e)
{
	UNUSED(e);

	int x, y;
	QWidget *parent = parentWidget();

	x = (parent->width() - width()) / 2;
	y = (parent->height() / 2 - height()) / 2;
	move(x, y);
}

void PoiInfoView::keyPressEvent(QKeyEvent * e)
{
	UNUSED(e);
	hide();
}

void PoiInfoView::mousePressEvent(QMouseEvent *e)
{
	UNUSED(e);
	hide();
}

void PoiInfoView::setIcon(QPixmap const &pix)
{
	icon->setPixmap(pix);
}

void PoiInfoView::setTitle(QString str)
{
	title->setText(str);
}

void PoiInfoView::setDesc(QString str)
{
	desc->setText(str);
}

void PoiInfoView::setInfo(QString str)
{
	info->setText(str);
}

void PoiInfoView::show(void)
{
	int x, y;
	QWidget *parent = parentWidget();

	x = (parent->width() - width()) / 2;
	y = (parent->height() / 2 - height()) / 2;
	move(x, y);

	QVBox::show();
}
