/*
  pos_queue.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "qgmap.h"
#include "pos.h"
#include "pos_queue.h"
#include <stdlib.h>

PosQueue::PosQueue()
{
	queue.setAutoDelete(TRUE);
}

void PosQueue::enqueue(const Pos &pos)
{
	struct pos_info *item = new struct pos_info;

	item->distance = calc_distance(center, pos);
	item->pos = pos;

	queue.inSort(item);
}

Pos PosQueue::first()
{
	struct pos_info *item = queue.first();
	return item->pos;
}

void PosQueue::remove(const Pos &pos)
{
	struct pos_info *item = find(pos);
	if (item)
		queue.removeRef(item);
}

bool PosQueue::inQueue(const Pos &pos)
{
	return !!find(pos);
}

void PosQueue::setCenter(const Pos &center)
{
	this->center = center;

	struct pos_info *p;
	p = queue.first();
	while (p) {
		if (center.getZoom() != p->pos.getZoom()) {
			queue.remove();
			p = queue.current();
		} else {
			p->distance = calc_distance(p->pos, center);
			p = queue.next();
		}
	}
	queue.sort();
}

int PosQueue::calc_distance(const Pos &a, const Pos &b)
{
	unsigned int dx = a.getX() - b.getX();
	unsigned int dy = a.getY() - b.getY();

	return dx * dx + dy * dy;
}

struct PosQueue::pos_info *PosQueue::find(const Pos &pos)
{
	struct pos_info *p;
	p = queue.first();
	while (p) {
		if (p->pos == pos)
			return p;
		p = queue.next();
	}
	return NULL;
}

void PosQueue::clear()
{
	queue.clear();
}

int PosQueue::count()
{
	return queue.count();
}

void PosQueue::dump()
{
	struct pos_info *p;
	p = queue.first();
	int i = 0;
	while (p) {
		printf("id:%d dist:%d pos:%s\n",
			   i++, p->distance, p->pos.serialize().ascii());
		p = queue.next();
	}
}
