/*
  pos_queue.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __POS_QUEUE_H__
#define __POS_QUEUE_H__

#include <qlist.h>
#include "pos.h"
#include <stdio.h>

class PosQueue
{
  public:
	PosQueue();
	void enqueue(const Pos &pos);
	Pos first();
	void remove(const Pos &pos);
	bool inQueue(const Pos &pos);
	void setCenter(const Pos &center);
	void clear();
	int count();

	void dump();

  private:
	struct pos_info {
		int distance;
		Pos pos;
	};

	class sortedQueue : public QList<struct pos_info>
	{
	public:
		int compareItems(QCollection::Item a, QCollection::Item b) {
			struct pos_info *ia = (struct pos_info *)a;
			struct pos_info *ib = (struct pos_info *)b;
			return ia->distance - ib->distance;
		}
	};
	sortedQueue queue;

	static int sort_distance(const void *, const void *);

	Pos center;

	int calc_distance(const Pos &a, const Pos &b);
	struct pos_info *find(const Pos &pos);
};

#endif // __POS_QUEUE_H__
